unit Bangs;

interface
Uses
  Boxes,
  LoadModules,
  BoxApi,
  BoxFiles,
  MyTypes,
  Windows;


Procedure RegisterBangs;
Procedure UnRegisterBangs;

implementation

Function AddBangCommand(command: string; f: Pointer): Boolean; cdecl; external lsapidll;
Function RemoveBangCommand(command: string): Boolean; cdecl; external lsapidll;

Procedure LsBox_create(Caller:Integer;Args:pchar); cdecl;
Begin
  if GetToken(Args,1)<>'' then
    BuildBox(GetToken(Args,1),0);
End;

Procedure LsBox_Destroy(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;

  Begin
 Box := GetBoxFromName(Args);
 If (Box <> nil) then
   Begin
     if GetFlag(Box^.Flags,UseFX) then
       AnimateWindow(Box^.hMainWnd,Box^.FXDelay,AW_HIDE or Box^.FX);

     SetFlag(Box^.Flags,isDocked,false);
     DestroyWindow(Box^.hMainWnd);
     DestroyBox(Box);
   End
End;

Procedure LsBox_Show(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
  child: HWND;
Begin
  Box := GetBoxFromName(Args);
  If (Box <> nil) then
   Begin
    if GetFlag(Box^.Flags,UseFX) then
    begin
      AnimateWindow(Box^.hMainWnd,Box^.FXDelay,Box^.FX);
      SetLayeredWindowAttributes(Box^.hMainWnd, RGB($FF,$00,$FF), Box^.ActTrans, LWA_ALPHA or LWA_COLORKEY);
      child := GetWindow(Box^.hMainWnd,GW_CHILD);
      child := GetWindow(child,GW_HWNDLAST);
      While child<>0 do
       begin
         InvalidateRgn(child,0,true);
         child := GetWindow(child,GW_HWNDPREV);
       end
    end else
      ShowWindow(Box^.hMainWnd,SW_Show);
    SetFlag(Box^.Flags,Hidden,false);
   End;
End;

Procedure LsBox_Hide(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
begin
  Box := GetBoxFromName(Args);
  If (Box <> nil) then
   Begin
    if GetFlag(Box^.Flags,UseFX) then
      AnimateWindow(Box^.hMainWnd,Box^.FXDelay,AW_HIDE or Box^.FX)
    else
      ShowWindow(Box^.hMainWnd,SW_Hide);
    SetFlag(Box^.Flags,Hidden,true);
   End;
end;

Procedure LsBox_Toggle(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
Begin
  Box := GetBoxFromName(Args);
  If (Box <> nil) then
    If GetFlag(Box^.Flags,Hidden) then
      LsBox_Show(Caller,Args)
    Else
      LsBox_Hide(Caller,Args)
End;

Procedure LsBox_ResetPos(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
Begin
  Box := First_Box;
  While ( Box<>nil ) Do
  Begin
    Box^.BoxPtr := OpenBox(Box^.BoxFile);
    If ( Box^.BoxPtr <> nil ) then
    begin
      SetWindowPos(Box^.hMainWnd, 0, GetBoxInt(Box^.BoxPtr,'LsBoxX',0),
                   GetBoxInt(Box^.BoxPtr,'LsBoxY',0), 0, 0, SWP_NOSIZE+SWP_NOZORDER);

      Box^.BoxPtr := CloseBox(Box^.BoxPtr,false);
    end;
    Box := Box^.Next_Box;
  End;
End;

Function FindItemGroup(id: integer): Pointer;
Var
  Group: PGroup;
Begin
  Group := First_Group;
  While ((Group<>nil)AND(Group^.id<>id)) Do
    Group := Group^.Next_Group;
  FindItemGroup := Group
End;

Procedure Group_Show(Caller:Integer;Args:pchar); cdecl;
Var
  Group: PGroup;
Begin
  Group := FindItemGroup(Str2Int(Args));
  If (Group<>nil) Then
  Begin
    SetGroupState(Group,SW_Show);
    SetFlag(Group^.Flags,Hidden,false);
  End;
End;

Procedure Group_Hide(Caller:Integer;Args:pchar); cdecl;
Var
  Group: PGroup;
Begin
  Group := FindItemGroup(Str2Int(Args));
  If (Group<>nil) Then
  Begin
    SetGroupState(Group,SW_Hide);
    SetFlag(Group^.Flags,Hidden,true);
  End;
End;

Procedure Group_Toggle(Caller:Integer;Args:pchar); cdecl;
Var
  Group: PGroup;
Begin
  Group := FindItemGroup(Str2Int(Args));
  If (Group<>nil) Then
    If GetFlag(Group^.Flags,Hidden) then
      Group_Show(Caller,Args)
    Else
      Group_Hide(Caller,Args)
End;

Procedure Group_Allign(Caller:Integer;Args:pchar); cdecl;
Var
  Group: PGroup;
Begin
  Group := FindItemGroup(Str2Int(Args));
  If (Group<>nil) Then
    AllignGroup(Group);
End;

Procedure Move_Box(Caller:Integer;Args:pchar); cdecl;
var
  Box: PBox;
  TargetX,TargetY,
  Time,Steps,
  i: integer;
  Step_Width,Step_Height: real;

begin
 If (Args <> '') then
  begin
  Box := GetBoxFromName(GetToken(Args,1));
  TargetX := Normalize(Str2Int(GetToken(Args,2)),ScreenX);
  TargetY := Normalize(Str2Int(GetToken(Args,3)),ScreenY);
  Steps := Str2Int(GetToken(Args,4));
  Time := Str2Int(GetToken(Args,5));
  With Box^ do
  begin
    if ( Steps>0 ) then
    begin
      Step_Width  := (TargetX-Box^.xPos) / Steps;
      Step_Height := (TargetY-Box^.yPos) / Steps;
      For i := 1 to Steps do
       begin
         Box^.xPos := Round(Box^.xPos+Step_Width);
         Box^.yPos := Round(Box^.yPos+Step_Height);
         MoveDockedBoxes(Box);
         SetWindowPos(hMainWnd,0,Box^.xPos,Box^.yPos,0,0,SWP_NOSIZE+SWP_NOZORDER);
         Sleep (Time);
       end;
    end;
    //make sure that the old Coordinates are Updated in the CFG we need them next time
    Box^.xPos := TargetX;
    Box^.yPos := TargetY;
    //This Prevents Rounding Errors and handles the Steps=0 case
    SetWindowPos(hMainWnd,0,TargetX,TargetY,0,0,SWP_NOSIZE+SWP_NOZORDER);
    MoveDockedBoxes(Box);
    //InvalidateRgn(Box^.hMainWnd,0,true);
  end
 end;
end;

Procedure Save_Coordinates(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
  Rect: TRect;
  FileName: String;
  Datei: Text;
Begin
  Box := GetBoxFromName(GetToken(Args,1));
  If (Box<>nil) Then
    With Box^ do
      begin
        FileName := GetToken(Args,2);
        if ( FileName<>'' ) Then
        begin
          AssignFile(Datei,FileName);
          Rewrite(Datei);
          Writeln(Datei,'LsBoxX 0');
          Writeln(Datei,'LsBoxY 0');
          Writeln(Datei);
          CloseFile(Datei);
          BoxPtr := OpenBox(FileName);
        end else
          BoxPtr := OpenBox(BoxFile);
        If ( BoxPtr <> nil ) then
        begin
          GetWindowRect(hMainWnd,Rect);
          SetBoxInt(BoxPtr,'LsBoxX',Rect.left MOD ScreenX);
          SetBoxInt(BoxPtr,'LsBoxY',Rect.top MOD ScreenY);
          BoxPtr := CloseBox(BoxPtr,true);
        end;
      end
End;

Procedure Set_Transparency(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
Begin
  Box := GetBoxFromName(GetToken(Args,1));
  If GetFlag(Box^.Flags, AlphaTransparent)AND(@SetLayeredWindowAttributes <> nil) Then
   With Box^ do
    begin
      Box^.ActTrans := Str2Int(GetToken(Args,2));
      If (GetToken(Args,3)<>'') then
        Box^.InActTrans := Str2Int(GetToken(Args,3))
      else
        Box^.InActTrans := Box^.ActTrans;
      InvalidateRgn(Box^.hMainWnd,0,true);
      SetActiveWindow(hMainWnd);
    end;
End;

Procedure Resize_Box(Caller:Integer;Args:pchar); cdecl;
Var
  Box: PBox;
Begin
  Box := GetBoxFromName(GetToken(Args,1));
  If (Box<>nil) Then
   With Box^ do
    begin
      If GetFlag(Box^.Flags,Resizeable) then
      begin
        Width := Str2Int(GetToken(Args,2));
        Height := Str2Int(GetToken(Args,3));
        BuildBackground(BackSkin, Border.left, Border.Top,
                        Border.Right, Border.Bottom,
                        Box^.hMainWnd, Box^.BackDc,Width,Height);
        //MoveWindow(hMainWnd,xPos,yPos,Width,Height,true);
        SetWindowPos(hMainWnd, 0, 0, 0, Width, Height, SWP_NOZORDER+SWP_NOMOVE);
      end;
    end
end;

Procedure Dock_me(Caller:Integer;Args:pchar); cdecl;
var
  Box1,Box2: PBox;
begin
  Box1 := GetBoxFromName(GetToken(Args,1));
  Box2 := GetBoxFromName(GetToken(Args,2));
  If ( Box1<>nil )AND(Box2<>nil) then
    SendMessage(Box1^.hMainWnd,BM_DOCK,Box2^.hMainWnd,(Box2^.yPos*$10000)+Box2^.xPos);
end;

Procedure UnDock_me(Caller:Integer;Args:pchar); cdecl;
var
  Box1,Box2: PBox;
begin
  Box1 := GetBoxFromName(GetToken(Args,1));
  Box2 := GetBoxFromName(GetToken(Args,2));
  If ( Box1<>nil )AND(Box2<>nil) then
    SendMessage(Box1^.hMainWnd,BM_UNDOCK,Box2^.hMainWnd,0);
end;

Procedure Box_Hook(Caller:Integer;Args:pchar); cdecl;
var
  Box: PBox;
  boxwnd: Hwnd;
begin
  boxwnd := Str2Int(StrRScan(Args,#32));
  Box := BuildBox(GetToken(Args,1),boxwnd);
  SetFlag(Box^.Flags,isDocked,true);
end;

Procedure Set_ZOrder(Caller:Integer;Args:pchar); cdecl;
var
  Box: PBox;

begin
  Box := GetBoxFromName(GetToken(Args,1));
  If (Box<>nil) Then
   If UpStr(GetToken(Args,2))='NORMAL' then
    begin
      SetFlag(Box^.Flags,HugBottom,false);
      SetWindowPos(Box^.hMainWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOMOVE);
    end else
     If UpStr(GetToken(Args,2))='STAYONTOP' then
       begin
         SetFlag(Box^.Flags,HugBottom,false);
         SetWindowPos(Box^.hMainWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOMOVE)
       end else
       If UpStr(GetToken(Args,2))='HUGBOTTOM' then
        begin
          SetWindowPos(Box^.hMainWnd, LSDskWnd, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOMOVE);
          SetFlag(Box^.Flags,HugBottom,true)
        end else
          Msgbox(MB_ICONASTERISK,'ZOrder must be either NORMAL, STAYONTOP or HUGBOTTOM');
end;

Procedure RefreshBox(Caller:Integer;Args:pchar); cdecl;
Var
  fBox: PBox;

 Procedure _RefreshBox(BoxName:string);
 var
   Box: PBox;
   Buffer: String;
   BackBMP: HBitmap;
   BHeight,
   BWidth: integer;
   tmpRGN: HRGN;
 begin
   Box := GetBoxFromName(GetToken(BoxName,1));
   If (Box<>nil) Then
    With Box^ do
     begin
       BoxPtr := OpenBox(BoxFile);
       //ReRead the Background BMP
       BackBMP := LoadLSImage(PChar(GetBoxString(BoxPtr,'LsBoxBackGround','LsBox.bmp')),nil);
       GetLSBitmapSize(BackBMP,BWidth,BHeight);

       //ReRead Hight and Width
       Height := GetBoxInt(BoxPtr,'LsBoxHeight',BHeight);
       Width := GetBoxInt(BoxPtr,'LsBoxWidth',BWidth);

       Buffer := GetBoxString(BoxPtr,'LsBoxDragRect','0 0 0 0');
       DragRGN := CreateRectRgn(Normalize(Str2Int(GetToken(buffer,1)),Width),
                                Normalize(Str2Int(GetToken(buffer,2)),Height),
                                Normalize(Str2Int(GetToken(buffer,3)),Width),
                                Normalize(Str2Int(GetToken(buffer,4)),Height));

       Buffer := BoxReadNextConfig(Box^.BoxPtr,'*LsBoxDragRect');
       While ( Buffer<>'' ) Do
        begin
          tmpRGN := CreateRectRgn(Normalize(Str2Int(GetToken(buffer,1)),Width),
                                  Normalize(Str2Int(GetToken(buffer,2)),Height),
                                  Normalize(Str2Int(GetToken(buffer,3)),Width),
                                  Normalize(Str2Int(GetToken(buffer,4)),Height));
          CombineRgn(dragRGN,dragRGN,tmpRGN,RGN_OR);
          DeleteObject(tmpRGN);
        
          Buffer := BoxReadNextConfig(Box^.BoxPtr,'*LsBoxDragRect');
        end;

       If GetFlag(Flags,Resizeable) Then
         DeleteObject(BackSkin);

       Buffer :=GetBoxString(BoxPtr,'LsBoxBackgroundOffs','0 0 0 0');
       If ((Buffer<>'0 0 0 0')and(Length(Buffer)>6)) then
        begin
          Border.left   := Str2Int(GetToken(Buffer,1));
          Border.Top    := Str2Int(GetToken(Buffer,2));
          Border.Right  := Str2Int(GetToken(Buffer,3));
          Border.Bottom := Str2Int(GetToken(Buffer,4));

          If (Border.left+Border.Right>=Width)then
           begin
             Border.left := 1;
             Border.Right := 1;
             MsgBox(MB_ICONWARNING,'Left or Right Border too wide for given image size!');
           end;

          If (Border.Top+Border.Bottom>=Height)then
           begin
             Border.top := 1;
             Border.bottom := 1;
             MsgBox(MB_ICONWARNING,'Top or Bottom Border too wide for given image size!');
           end;

          BuildBackground(BackBMP, Border.left, Border.Top,
                          Border.Right, Border.Bottom,
                          hMainWnd, BackDc,Width,Height);

          SetFlag(Flags,Resizeable,true);
          Box^.BackSkin := BackBMP
       end else Begin
         SelectObject(Box^.BackDc, BackBMP);
         SetFlag(Flags,Resizeable,false);
         DeleteObject(BackBMP);
       end;

       LeftClickBang := GetBoxString(BoxPtr,'LsBoxOnLeftClick','!none');
       RightClickBang := GetBoxString(BoxPtr,'LsBoxOnRightClick','!popup');

       If GetBoxBool(Box^.BoxPtr,'LsBoxAutoHide') then
         SetFlag(Box^.Flags, AutoHide, true);

       BoxPtr := CloseBox(Box^.BoxPtr,false);

       SetWindowPos(hMainWnd, 0, 0, 0, Width, Height, SWP_NOZORDER+SWP_NOMOVE);

       //UnLoadLoadModules(First_LoadModule);
       //First_LoadModule := nil;
       //LoadLoadModules(Box);
       {//Remove Shortcuts
       DestroyShortcuts(First_Shortcut);
       First_Shortcut := nil;

       //Reload Shortcuts
       BuildShortcuts(Box);}

       InvalidateRect(hMainWnd,nil,true);
     end;
 end;

Begin
  fBox := First_Box;
  if UpStr(GetToken(Args,1))='ALL_BOXES' then
   While ( fBox<>nil ) Do
    begin
      _Refreshbox(fBox.Title);
      fBox := fBox^.Next_Box;
    end
  else
    _Refreshbox(Args);
End;

Procedure RedrawBox(Caller:Integer;Args:pchar); cdecl;
var
  Box: PBox;
begin
  Box := First_Box;
  if UpStr(GetToken(Args,1))='ALL_BOXES' then
   While ( Box<>nil ) Do
    begin
      InvalidateRect(Box^.hMainWnd,nil,true);
      Box := Box^.Next_Box;
    end
   else
    begin
      Box := GetBoxFromName(GetToken(Args,1));
      if (Box<>nil) then
        InvalidateRect(Box^.hMainWnd,nil,true);
    end;
end;


Procedure RegisterBangs;
begin
  AddBangCommand('!LsBoxCreate',@LsBox_create);
  AddBangCommand('!LsBoxDestroy',@LsBox_Destroy);
  AddBangCommand('!LsBoxHide',@LsBox_Hide);
  AddBangCommand('!LsBoxShow',@LsBox_Show);
  AddBangCommand('!LsBoxToggle',@LsBox_Toggle);
  AddBangCommand('!LsBoxResetPos',@LsBox_ResetPos);
  AddBangCommand('!LsBoxGroupShow',@Group_Show);
  AddBangCommand('!LsBoxGroupHide',@Group_Hide);
  AddBangCommand('!LsBoxGroupToggle',@Group_Toggle);
  AddBangCommand('!LsBoxGroupAllign',@Group_Allign);
  AddBangCommand('!LsBoxMoveBox',@Move_Box);
  AddBangCommand('!LsBoxSavePosition',@Save_Coordinates);
  AddBangCommand('!LsBoxSetTransparency',@Set_Transparency);
  AddBangCommand('!LsBoxResize',@Resize_Box);
  AddBangCommand('!LsBoxBoxHook',@Box_Hook);
  AddBangCommand('!LsBoxSetZOrder',@Set_ZOrder);
  AddBangCommand('!LsBoxRefreshBox',@RefreshBox);
 //   -{Bangs} Added "!LsBoxRefreshBox" - This will reread most Box-specific settings and
 //            update the box accordingly.  Use ALL_BOXES to apply it to every Box.
  AddBangCommand('!LsBoxRedrawBox',@RedrawBox);
  AddBangCommand('!LsBoxDock',@Dock_me);
  AddBangCommand('!LsBoxUnDock',@UnDock_me);
end;

Procedure UnRegisterBangs;
begin
  RemoveBangCommand('!LsBoxCreate');
  RemoveBangCommand('!LsBoxDestroy');
  RemoveBangCommand('!LsBoxHide');
  RemoveBangCommand('!LsBoxShow');
  RemoveBangCommand('!LsBoxToggle');
  RemoveBangCommand('!LsBoxResetPos');
  RemoveBangCommand('!LsBoxGroupHide');
  RemoveBangCommand('!LsBoxGroupShow');
  RemoveBangCommand('!LsBoxGroupToggle');
  RemoveBangCommand('!LsBoxGroupAllign');
  RemoveBangCommand('!LsBoxMoveBox');
  RemoveBangCommand('!LsBoxSavePosition');
  RemoveBangCommand('!LsBoxSetTransparency');
  RemoveBangCommand('!LsBoxResize');
  RemoveBangCommand('!LsBoxBoxHook');
  RemoveBangCommand('!LsBoxSetZOrder');
  RemoveBangCommand('!LsBoxRefreshBox');
  RemoveBangCommand('!LsBoxRedrawBox');
  RemoveBangCommand('!LsBoxDock');
  RemoveBangCommand('!LsBoxUnDock');
end;

end.
